==> GotoBLAS-1.10.tar.gz: GotoBLAS/level3/gemm/gemm.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "common.h"

#ifdef XDOUBLE
#ifndef COMPLEX
#define ERROR_NAME "QGEMM "
#else
#define ERROR_NAME "XGEMM "
#endif
#elif defined(DOUBLE)
#ifndef COMPLEX
#define ERROR_NAME "DGEMM "
#else
#define ERROR_NAME "ZGEMM "
#endif
#else
#ifndef COMPLEX
#define ERROR_NAME "SGEMM "
#else
#define ERROR_NAME "CGEMM "
#endif
#endif

static GEMMRETTYPE (*gemm[])(long, long, long, FLOAT,
			     FLOAT *, long,
			     FLOAT* , long,
			     FLOAT *, long, FLOAT *) = {
			       GEMM_NN, GEMM_TN, GEMM_NT, GEMM_TT,
			     };

GEMMRETTYPE NAME(char *TRANSA, char *TRANSB,
		  blasint *M, blasint *N, blasint *K,
		  FLOAT *ALPHA,
		  FLOAT *a, blasint *ldA,
		  FLOAT *b, blasint *ldB,
		  FLOAT *BETA,
		  FLOAT *c, blasint *ldC){
  
  int transa, transb, nrowa, nrowb;
  blasint info;
  blasint lda, ldb, ldc, m, n, k;
  FLOAT alpha, beta;
  char transA, transB;
  FLOAT *buffer;

#ifdef SMP
  int mode;
#endif

#ifdef COPYOVERHEAD
  copyoverhead_t copyoverhead;
#define OVERHEAD copyoverhead
#else
#define OVERHEAD 0
#endif

  lda = *ldA;
  ldb = *ldB;
  ldc = *ldC;

  m   = *M;
  n   = *N;
  k   = *K;
  transA = *TRANSA;
  transB = *TRANSB;

  alpha = ALPHA[0];
  beta  = BETA[0];

  TOUPPER(transA);
  TOUPPER(transB);

  transa = -1;
  transb = -1;

  if (transA == 'N') transa = 0;
  if (transA == 'T') transa = 1;
  if (transA == 'R') transa = 0;
  if (transA == 'C') transa = 1;

  if (transB == 'N') transb = 0;
  if (transB == 'T') transb = 1;
  if (transB == 'R') transb = 0;
  if (transB == 'C') transb = 1;

  nrowa = m;
  if (transa) nrowa = k;
  nrowb = k;
  if (transb) nrowb = n;

/* Test the input parameters. */
  info = 0;

  if (ldc < m)     info = 13;
  if (ldb < nrowb) info = 10;
  if (lda < nrowa) info = 8; 
  if (k < 0)       info = 5;
  if (n < 0)       info = 4;
  if (m < 0)       info = 3;
  if (transb < 0)  info = 2;
  if (transa < 0)  info = 1;

  if (info){
    BLASFUNC(xerbla)(ERROR_NAME, &info, sizeof(ERROR_NAME));
    return OVERHEAD;
  }

  if ((m == 0) || (n == 0)) return OVERHEAD;

#ifdef SMP
  if (blas_cpu_number == 0) blas_get_cpu_number();
#ifdef SMP_SERVER
  if (blas_server_avail == 0) blas_thread_init();
#endif
#endif

#ifndef SMP
  if (beta != ONE) GEMM_BETA(m, n, 0, beta, NULL, 0, NULL, 0, c, ldc);
#else
#ifdef DOUBLE
  mode  =  BLAS_DOUBLE | BLAS_REAL;
#else
  mode  =  BLAS_SINGLE | BLAS_REAL;
#endif  

  if (beta != ONE) 
    beta_thread(mode, m, n, beta, ZERO, c, ldc, (void *)GEMM_BETA);
#endif

#ifndef COPYOVERHEAD
  if (k == 0) return 0;

  if (alpha == ZERO) return 0;
#else
  if (k == 0) return copyoverhead;

  if (alpha == ZERO) return copyoverhead;
#endif
  
  buffer = (FLOAT *)blas_memory_alloc(0);

#ifndef SMP

#ifdef COPYOVERHEAD
  copyoverhead =
#endif
  (gemm[(transb << 1) | transa])
    (m, n, k, alpha, a, lda, b, ldb, c, ldc, buffer);

#else
  mode |= (transa << BLAS_TRANSA_SHIFT);
  mode |= (transb << BLAS_TRANSB_SHIFT);

  gemm_thread_mn(mode, m, n, k, alpha, ALPHA[1],
	      a, lda, b, ldb, c, ldc, 
	      (void *)gemm[(transb << 1) | transa], buffer);

#endif

  blas_memory_free(buffer);


  return OVERHEAD;
}

==> GotoBLAS-1.12.tar.gz: GotoBLAS/level3/gemm/gemm.c <==
/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "common.h"

#ifdef XDOUBLE
#ifndef COMPLEX
#define ERROR_NAME "QGEMM "
#else
#define ERROR_NAME "XGEMM "
#endif
#elif defined(DOUBLE)
#ifndef COMPLEX
#define ERROR_NAME "DGEMM "
#else
#define ERROR_NAME "ZGEMM "
#endif
#else
#ifndef COMPLEX
#define ERROR_NAME "SGEMM "
#else
#define ERROR_NAME "CGEMM "
#endif
#endif

static GEMMRETTYPE (*gemm[])(long, long, long, FLOAT,
			     FLOAT *, long,
			     FLOAT* , long,
			     FLOAT *, long, FLOAT *) = {
			       GEMM_NN, GEMM_TN, GEMM_NT, GEMM_TT,
			     };

GEMMRETTYPE NAME(char *TRANSA, char *TRANSB,
		  blasint *M, blasint *N, blasint *K,
		  FLOAT *ALPHA,
		  FLOAT *a, blasint *ldA,
		  FLOAT *b, blasint *ldB,
		  FLOAT *BETA,
		  FLOAT *c, blasint *ldC){
  
  int transa, transb, nrowa, nrowb;
  blasint info;
  blasint lda, ldb, ldc, m, n, k;
  FLOAT alpha, beta;
  char transA, transB;
  FLOAT *buffer;

#ifdef SMP
  int mode;
#endif

#ifdef COPYOVERHEAD
  copyoverhead_t copyoverhead;
#define OVERHEAD copyoverhead
#else
#define OVERHEAD 0
#endif

  lda = *ldA;
  ldb = *ldB;
  ldc = *ldC;

  m   = *M;
  n   = *N;
  k   = *K;
  transA = *TRANSA;
  transB = *TRANSB;

  alpha = ALPHA[0];
  beta  = BETA[0];

  TOUPPER(transA);
  TOUPPER(transB);

  transa = -1;
  transb = -1;

  if (transA == 'N') transa = 0;
  if (transA == 'T') transa = 1;
  if (transA == 'R') transa = 0;
  if (transA == 'C') transa = 1;

  if (transB == 'N') transb = 0;
  if (transB == 'T') transb = 1;
  if (transB == 'R') transb = 0;
  if (transB == 'C') transb = 1;

  nrowa = m;
  if (transa) nrowa = k;
  nrowb = k;
  if (transb) nrowb = n;

/* Test the input parameters. */
  info = 0;

  if (ldc < m)     info = 13;
  if (ldb < nrowb) info = 10;
  if (lda < nrowa) info = 8; 
  if (k < 0)       info = 5;
  if (n < 0)       info = 4;
  if (m < 0)       info = 3;
  if (transb < 0)  info = 2;
  if (transa < 0)  info = 1;

  if (info){
    BLASFUNC(xerbla)(ERROR_NAME, &info, sizeof(ERROR_NAME));
    return OVERHEAD;
  }

  if ((m == 0) || (n == 0)) return OVERHEAD;

#ifdef SMP
  if (blas_cpu_number == 0) blas_get_cpu_number();
#ifdef SMP_SERVER
  if (blas_server_avail == 0) blas_thread_init();
#endif
#endif

#ifndef SMP
  if (beta != ONE) GEMM_BETA(m, n, 0, beta, NULL, 0, NULL, 0, c, ldc);
#else

#ifdef XDOUBLE
  mode  =  BLAS_XDOUBLE | BLAS_REAL;
#elif defined(DOUBLE)
  mode  =  BLAS_DOUBLE  | BLAS_REAL;
#else
  mode  =  BLAS_SINGLE  | BLAS_REAL;
#endif  

  if (beta != ONE) 
    beta_thread(mode, m, n, beta, ZERO, c, ldc, (void *)GEMM_BETA);
#endif

#ifndef COPYOVERHEAD
  if (k == 0) return 0;

  if (alpha == ZERO) return 0;
#else
  if (k == 0) return copyoverhead;

  if (alpha == ZERO) return copyoverhead;
#endif
  
  buffer = (FLOAT *)blas_memory_alloc(0);

#ifndef SMP

#ifdef COPYOVERHEAD
  copyoverhead =
#endif
  (gemm[(transb << 1) | transa])
    (m, n, k, alpha, a, lda, b, ldb, c, ldc, buffer);

#else
  mode |= (transa << BLAS_TRANSA_SHIFT);
  mode |= (transb << BLAS_TRANSB_SHIFT);

  GEMM_THREAD(mode, m, n, k, alpha, ALPHA[1],
	      a, lda, b, ldb, c, ldc, 
	      (void *)gemm[(transb << 1) | transa], buffer);

#endif

  blas_memory_free(buffer);


  return OVERHEAD;
}
